﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2015 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include "SivPointVector.hpp"
# include "SivMath.hpp"

namespace s3d
{
	/// <summary>
	/// カメラ
	/// </summary>
	struct Camera
	{
		/// <summary>
		/// カメラの位置
		/// </summary>
		Vec3 pos = { 0.0, 4.0, -16.0 };

		/// <summary>
		/// カメラの注視点
		/// </summary>
		Vec3 lookat = { 0.0, 1.0, 0.0 };

		/// <summary>
		/// カメラの上方向のベクトル
		/// </summary>
		Vec3 up = { 0.0, 1.0, 0.0 };

		/// <summary>
		/// 視野角（°）
		/// </summary>
		double fovDegree = 45.0;

		/// <summary>
		/// ニアクリップ
		/// </summary>
		double nearclip = 0.1;

		/// <summary>
		/// デフォルトコンストラクタ
		/// </summary>
		Camera() {}

		/// <summary>
		/// カメラの設定を作成します。
		/// </summary>
		/// <param name="_pos">
		/// カメラの位置
		/// </param>
		/// <param name="_lookat">
		/// カメラの注視点
		/// </param>
		/// <param name="_up">
		/// カメラの上方向のベクトル
		/// </param>
		/// <param name="_fovDegree">
		/// 視野角（°）
		/// </param>
		/// <param name="_nearclip">
		/// ニアクリップ
		/// </param>
		Camera(const Vec3& _pos, const Vec3& _lookat, const Vec3& _up, double _fovDegree, double _nearclip)
			: pos(_pos)
			, lookat(_lookat)
			, up(_up)
			, fovDegree(_fovDegree)
			, nearclip(_nearclip) {}

		/// <summary>
		/// 視野角をラジアンで取得します。
		/// </summary>
		/// <returns>
		/// 視野角（ラジアン）
		/// </returns>
		double fovRadian() const
		{
			return Math::Radians(fovDegree);
		}
	};
}
