﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2015 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include <wchar.h>
# include "SivTypes.hpp"
# define HalfWidthSpace L' '
# define FullWidthSpace L'　'

namespace s3d
{
	/// <summary>
	/// 10 進数の数字であるかを示します。
	/// </summary>
	inline bool IsDigit(uint32 ch)
	{
		return (ch - L'0') <= (L'9' - L'0');
	}

	/// <summary>
	/// アルファベットの小文字であるかを示します。
	/// </summary>
	inline bool IsLower(uint32 ch)
	{
		return (ch - L'a') <= (L'z' - L'a');
	}

	/// <summary>
	/// アルファベットの大文字であるかを示します。
	/// </summary>
	inline bool IsUpper(uint32 ch)
	{
		return (ch - L'A') <= (L'Z' - L'A');
	}

	/// <summary>
	/// アルファベットであるかを示します。
	/// </summary>
	inline bool IsAlpha(uint32 ch)
	{
		return IsLower(ch) || IsUpper(ch);
	}

	/// <summary>
	/// アルファベットもしくは数字であるかを示します。
	/// </summary>
	inline bool IsAlnum(uint32 ch)
	{
		return IsDigit(ch) || IsAlpha(ch);
	}

	/// <summary>
	/// 16 進数の数字であるかを示します。
	/// </summary>
	inline bool IsXdigit(uint32 ch)
	{
		return IsDigit(ch) || ((ch - L'A') <= (L'F' - L'A')) || ((ch - L'a') <= (L'f' - L'a'));
	}

	/// <summary>
	/// 制御文字であるかを示します。
	/// </summary>
	inline bool IsControl(uint32 ch)
	{
		return (ch <= 0x1F) || ((ch - 0x7F) <= (0x9F - 0x7F));
	}

	/// <summary>
	/// 空白文字であるかを示します。
	/// </summary>
	/// <remarks>
	/// L' ', L'\t', L'　' が該当します。
	/// </remarks>
	inline bool IsBlank(uint32 ch)
	{
		return (ch == HalfWidthSpace) || (ch == L'\t') || (ch == FullWidthSpace);
	}

	/// <summary>
	/// 空白類文字であるかを示します。
	/// </summary>
	/// <remarks>
	/// L' ', L'\t', L'\n', L'\v', L'\f', L'\r', L'　' が該当します。
	/// </remarks>
	inline bool IsSpace(uint32 ch)
	{
		return ((ch - L'\t') <= (L'\r' - L'\t')) || (ch == HalfWidthSpace) || (ch == FullWidthSpace);
	}

	/// <summary>
	/// 印字可能文字であるかを示します。
	/// </summary>
	inline bool IsPrint(uint32 ch)
	{
		return !!::iswprint(static_cast<wchar_t>(ch));
	}

	/// <summary>
	/// 文字
	/// </summary>
	struct Char
	{
		uint32 ch;

		/// <summary>
		/// コンストラクタ
		/// </summary>
		Char() = default;

		/// <summary>
		/// コンストラクタ
		/// </summary>
		/// <param name="_ch">
		/// コピーする文字
		/// </param>
		Char(uint32 _ch)
			: ch{ _ch } {}
	};
}

# undef HalfWidthSpace
# undef FullWidthSpace
