﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2015 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include "Siv3DFwd.hpp"
# include "SivArray.hpp"
# include "SivString.hpp"
# include "SivMemoryReader.hpp"

namespace s3d
{
	/// <summary>
	/// AES-128 暗号鍵
	/// </summary>
	union AES128Key
	{
		/// <summary>
		/// デフォルトコンストラクタ
		/// </summary>
		AES128Key() = default;

		/// <summary>
		/// AES-128 暗号鍵を定義します。
		/// </summary>
		/// <param name="a">
		/// 8 bit 整数
		/// </param>
		/// <param name="b">
		/// 8 bit 整数
		/// </param>
		/// <param name="c">
		/// 8 bit 整数
		/// </param>
		/// <param name="d">
		/// 8 bit 整数
		/// </param>
		/// <param name="e">
		/// 8 bit 整数
		/// </param>
		/// <param name="f">
		/// 8 bit 整数
		/// </param>
		/// <param name="g">
		/// 8 bit 整数
		/// </param>
		/// <param name="h">
		/// 8 bit 整数
		/// </param>
		/// <param name="i">
		/// 8 bit 整数
		/// </param>
		/// <param name="j">
		/// 8 bit 整数
		/// </param>
		/// <param name="k">
		/// 8 bit 整数
		/// </param>
		/// <param name="l">
		/// 8 bit 整数
		/// </param>
		/// <param name="m">
		/// 8 bit 整数
		/// </param>
		/// <param name="n">
		/// 8 bit 整数
		/// </param>
		/// <param name="o">
		/// 8 bit 整数
		/// </param>
		/// <param name="p">
		/// 8 bit 整数
		/// </param>
		AES128Key(uint8 a, uint8 b, uint8 c, uint8 d, uint8 e, uint8 f, uint8 g, uint8 h
				, uint8 i, uint8 j, uint8 k, uint8 l, uint8 m, uint8 n, uint8 o, uint8 p)
		{
			v8[0] = a;
			v8[1] = b;
			v8[2] = c;
			v8[3] = d;
			v8[4] = e;
			v8[5] = f;
			v8[6] = g;
			v8[7] = h;

			v8[8] = i;
			v8[9] = j;
			v8[10] = k;
			v8[11] = l;
			v8[12] = m;
			v8[13] = n;
			v8[14] = o;
			v8[15] = p;
		}

		/// <summary>
		/// AES-128 暗号鍵を定義します。
		/// </summary>
		/// <param name="a">
		/// 16 bit 整数
		/// </param>
		/// <param name="b">
		/// 16 bit 整数
		/// </param>
		/// <param name="c">
		/// 16 bit 整数
		/// </param>
		/// <param name="d">
		/// 16 bit 整数
		/// </param>
		/// <param name="e">
		/// 16 bit 整数
		/// </param>
		/// <param name="f">
		/// 16 bit 整数
		/// </param>
		/// <param name="g">
		/// 16 bit 整数
		/// </param>
		/// <param name="h">
		/// 16 bit 整数
		/// </param>
		AES128Key(uint16 a, uint16 b, uint16 c, uint16 d, uint16 e, uint16 f, uint16 g, uint16 h)
		{
			v16[0] = a;
			v16[1] = b;
			v16[2] = c;
			v16[3] = d;
			v16[4] = e;
			v16[5] = f;
			v16[6] = g;
			v16[7] = h;
		}

		/// <summary>
		/// AES-128 暗号鍵を定義します。
		/// </summary>
		/// <param name="a">
		/// 32 bit 整数
		/// </param>
		/// <param name="b">
		/// 32 bit 整数
		/// </param>
		/// <param name="c">
		/// 32 bit 整数
		/// </param>
		/// <param name="d">
		/// 32 bit 整数
		/// </param>
		AES128Key(uint32 a, uint32 b, uint32 c, uint32 d)
		{
			v32[0] = a;
			v32[1] = b;
			v32[2] = c;
			v32[3] = d;
		}

		/// <summary>
		/// AES-128 暗号鍵を定義します。
		/// </summary>
		/// <param name="a">
		/// 64 bit 整数
		/// </param>
		/// <param name="b">
		/// 64 bit 整数
		/// </param>
		AES128Key(uint64 a, uint64 b)
		{
			v64[0] = a;
			v64[1] = b;
		}

		uint8 v8[16];

		uint16 v16[8];

		uint32 v32[4];

		uint64 v64[2];
	};

	/// <summary>
	/// AES-128 初期化ベクトル
	/// </summary>
	union AESIV
	{
		/// <summary>
		/// デフォルトコンストラクタ
		/// </summary>
		AESIV() = default;

		/// <summary>
		/// AES-128 初期化ベクトルを定義します。
		/// </summary>
		/// <param name="a">
		/// 8 bit 整数
		/// </param>
		/// <param name="b">
		/// 8 bit 整数
		/// </param>
		/// <param name="c">
		/// 8 bit 整数
		/// </param>
		/// <param name="d">
		/// 8 bit 整数
		/// </param>
		/// <param name="e">
		/// 8 bit 整数
		/// </param>
		/// <param name="f">
		/// 8 bit 整数
		/// </param>
		/// <param name="g">
		/// 8 bit 整数
		/// </param>
		/// <param name="h">
		/// 8 bit 整数
		/// </param>
		/// <param name="i">
		/// 8 bit 整数
		/// </param>
		/// <param name="j">
		/// 8 bit 整数
		/// </param>
		/// <param name="k">
		/// 8 bit 整数
		/// </param>
		/// <param name="l">
		/// 8 bit 整数
		/// </param>
		/// <param name="m">
		/// 8 bit 整数
		/// </param>
		/// <param name="n">
		/// 8 bit 整数
		/// </param>
		/// <param name="o">
		/// 8 bit 整数
		/// </param>
		/// <param name="p">
		/// 8 bit 整数
		/// </param>
		AESIV(uint8 a, uint8 b, uint8 c, uint8 d, uint8 e, uint8 f, uint8 g, uint8 h
			, uint8 i, uint8 j, uint8 k, uint8 l, uint8 m, uint8 n, uint8 o, uint8 p)
		{
			v8[0] = a;
			v8[1] = b;
			v8[2] = c;
			v8[3] = d;
			v8[4] = e;
			v8[5] = f;
			v8[6] = g;
			v8[7] = h;

			v8[8] = i;
			v8[9] = j;
			v8[10] = k;
			v8[11] = l;
			v8[12] = m;
			v8[13] = n;
			v8[14] = o;
			v8[15] = p;
		}

		/// <summary>
		/// AES-128 初期化ベクトルを定義します。
		/// </summary>
		/// <param name="a">
		/// 16 bit 整数
		/// </param>
		/// <param name="b">
		/// 16 bit 整数
		/// </param>
		/// <param name="c">
		/// 16 bit 整数
		/// </param>
		/// <param name="d">
		/// 16 bit 整数
		/// </param>
		/// <param name="e">
		/// 16 bit 整数
		/// </param>
		/// <param name="f">
		/// 16 bit 整数
		/// </param>
		/// <param name="g">
		/// 16 bit 整数
		/// </param>
		/// <param name="h">
		/// 16 bit 整数
		/// </param>
		AESIV(uint16 a, uint16 b, uint16 c, uint16 d, uint16 e, uint16 f, uint16 g, uint16 h)
		{
			v16[0] = a;
			v16[1] = b;
			v16[2] = c;
			v16[3] = d;
			v16[4] = e;
			v16[5] = f;
			v16[6] = g;
			v16[7] = h;
		}

		/// <summary>
		/// AES-128 初期化ベクトルを定義します。
		/// </summary>
		/// <param name="a">
		/// 32 bit 整数
		/// </param>
		/// <param name="b">
		/// 32 bit 整数
		/// </param>
		/// <param name="c">
		/// 32 bit 整数
		/// </param>
		/// <param name="d">
		/// 32 bit 整数
		/// </param>
		AESIV(uint32 a, uint32 b, uint32 c, uint32 d)
		{
			v32[0] = a;
			v32[1] = b;
			v32[2] = c;
			v32[3] = d;
		}

		/// <summary>
		/// AES-128 初期化ベクトルを定義します。
		/// </summary>
		/// <param name="a">
		/// 64 bit 整数
		/// </param>
		/// <param name="b">
		/// 64 bit 整数
		/// </param>
		AESIV(uint64 a, uint64 b)
		{
			v64[0] = a;
			v64[1] = b;
		}

		uint8 v8[16];

		uint16 v16[8];

		uint32 v32[4];

		uint64 v64[2];

		/// <summary>
		/// デフォルトの AES-128 初期化ベクトルを返します。
		/// </summary>
		/// <return>
		/// デフォルトの AES-128 初期化ベクトル
		/// </return>
		static AESIV Default()
		{
			return AESIV(0, 0, 0, 0);
		}
	};

	/// <summary>
	/// 暗号化
	/// </summary>
	/// <remarks>
	/// データを暗号化する機能を提供します。
	/// </remarks>
	namespace Crypto
	{
		/// <summary>
		/// データを暗号化します。
		/// </summary>
		/// <param name="data">
		/// 暗号化するデータの先頭ポインタ
		/// </param>
		/// <param name="size">
		/// 暗号化するデータのサイズ
		/// </param>
		/// <param name="key">
		/// 暗号鍵
		/// </param>
		/// <param name="iv">
		/// 初期化ベクトル
		/// </param>
		/// <returns>
		/// 暗号化されたデータ
		/// </returns>
		MemoryReader Encrypt(const void* data, size_t size, const AES128Key& key, const AESIV& iv = AESIV::Default());

		/// <summary>
		/// データを暗号化します。
		/// </summary>
		/// <param name="from">
		/// 暗号化するデータ
		/// </param>
		/// <param name="key">
		/// 暗号鍵
		/// </param>
		/// <param name="iv">
		/// 初期化ベクトル
		/// </param>
		/// <returns>
		/// 暗号化されたデータ
		/// </returns>
		inline MemoryReader Encrypt(const Array<uint8>& from, const AES128Key& key, const AESIV& iv = AESIV::Default())
		{
			return Encrypt(from.data(), from.size(), key, iv);
		}

		/// <summary>
		/// 文字列を暗号化します。
		/// </summary>
		/// <param name="str">
		/// 暗号化する文字列
		/// </param>
		/// <param name="key">
		/// 暗号鍵
		/// </param>
		/// <param name="iv">
		/// 初期化ベクトル
		/// </param>
		/// <returns>
		/// 暗号化されたデータ
		/// </returns>
		inline MemoryReader EncryptString(const String& str, const AES128Key& key, const AESIV& iv = AESIV::Default())
		{
			return Encrypt(str.data(), str.length * sizeof(wchar), key, iv);
		}

		/// <summary>
		/// データを暗号化します。
		/// </summary>
		/// <param name="data">
		/// 暗号化するデータ
		/// </param>
		/// <param name="iv">
		/// 初期化ベクトル
		/// </param>
		/// <returns>
		/// 暗号化されたデータ
		/// </returns>
		template<typename Type> inline MemoryReader Encrypt(const Type& data, const AES128Key& key, const AESIV& iv = AESIV::Default())
		{
			static_assert(std::is_trivially_copyable<Type>::value, "Type must be trivially copyable");

			return Encrypt(std::addressof(data), sizeof(data), key, iv);
		}

		/// <summary>
		/// 暗号化されたデータを復号します。
		/// </summary>
		/// <param name="from">
		/// 暗号化されたデータ
		/// </param>
		/// <param name="to">
		/// 復号したデータの出力先のバッファの先頭ポインタ
		/// </param>
		/// <param name="size">
		/// 出力先のバッファサイズ
		/// </param>
		/// <param name="key">
		/// 暗号鍵
		/// </param>
		/// <param name="iv">
		/// 初期化ベクトル
		/// </param>
		/// <returns>
		/// 復号に成功した場合 true, それ以外の場合は false
		/// </returns>
		bool Decrypt(const MemoryReader& from, void* to, size_t size, const AES128Key& key, const AESIV& iv = AESIV::Default());

		/// <summary>
		/// 暗号化された文字列を復号します。
		/// </summary>
		/// <param name="from">
		/// 暗号化されたデータ
		/// </param>
		/// <param name="to">
		/// 復号したデータの出力先
		/// </param>
		/// <param name="key">
		/// 暗号鍵
		/// </param>
		/// <param name="iv">
		/// 初期化ベクトル
		/// </param>
		/// <returns>
		/// 復号に成功した場合 true, それ以外の場合は false
		/// </returns>
		bool DecryptString(const MemoryReader& from, String& to, const AES128Key& key, const AESIV& iv = AESIV::Default());

		/// <summary>
		/// 暗号化されたデータを復号します。
		/// </summary>
		/// <param name="from">
		/// 暗号化されたデータ
		/// </param>
		/// <param name="to">
		/// 復号したデータの出力先
		/// </param>
		/// <param name="key">
		/// 暗号鍵
		/// </param>
		/// <param name="iv">
		/// 初期化ベクトル
		/// </param>
		/// <returns>
		/// 復号に成功した場合 true, それ以外の場合は false
		/// </returns>
		template<typename Type> inline bool Decrypt(const MemoryReader& from, Type& to, const AES128Key& key, const AESIV& iv = AESIV::Default())
		{
			static_assert(std::is_trivially_copyable<Type>::value, "Type must be trivially copyable");

			return Decrypt(from, std::addressof(to), sizeof(to), key, iv);
		}
	}
}
