﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2015 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include "SivGrid.hpp"
# include "SivCustomColor.hpp"
# include "SivImage.hpp"
# include "SivTextureFormat.hpp"

namespace s3d
{
	template<typename Type>
	class CustomImage : public Grid<Type>
	{
	public:

		using ColorType = Type;

		const static TextureFormat Format = Type::Format;

		CustomImage() = default;

		CustomImage(int32 width, int32 height)
			: Grid(width, height) {}

		CustomImage(int32 width, int32 height, const Type& value)
			: Grid(width, height, value) {}

		CustomImage(const FilePath& path);

		CustomImage(const Image& image)
		{
			fromImage(image);
		}

		/// <summary>
		/// 画像が空ではないかを返します。
		/// </summary>
		/// <returns>
		/// 画像が空ではない場合 true, それ以外の場合は false
		/// </returns>
		explicit operator bool() const
		{
			return !isEmpty;
		}

		/// <summary>
		/// 画像の幅と高さ（ピクセル）
		/// </summary>
		Property_Get(Size, size) const { return{ width, height }; }

		/// <summary>
		/// 画像の各行のデータサイズ
		/// </summary>
		Property_Get(uint32, stride) const { return width * sizeof(Type); }

		/// <summary>
		/// 画像のピクセル数
		/// </summary>
		Property_Get(uint32, num_pixels) const { return width * height; }

		/// <summary>
		/// 画像のデータサイズ
		/// </summary>
		Property_Get(uint32, dataSize) const { return stride * height; }

		Image toImage() const
		{
			if (isEmpty)
			{
				return Image();
			}

			Image image(width, height);
			Color* pDst = image[0];
			const Color* pDstEnd = pDst + image.num_pixels;
			const Type* pSrc = data();
			
			while (pDst != pDstEnd)
			{
				(*pDst++) = (pSrc++)->toColor();
			}

			return image;
		}

		void fromImage(const Image& image)
		{
			resize(image.width, image.height);

			if (!image)
			{
				return;
			}

			const Color* pSrc = image[0];
			const Color* pSrcEnd = pSrc + image.num_pixels;
			Type* pDst = data();

			while (pSrc != pSrcEnd)
			{
				(pDst++)->fromColor(*pSrc++);
			}
		}

		bool saveDDS(const FilePath& path) const;

		MemoryReader encodeDDS() const;
	};

	using ImageR32F		= CustomImage<R32F>;
	using ImageRGBA16F	= CustomImage<RGBA16F>;
	using ImageRGBA32F	= CustomImage<RGBA32F>;
}
