﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2015 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include <DirectXMath.h>
# include "SivPointVector.hpp"

namespace s3d
{
	union Float4A
	{
		__m128 v;

#pragma warning(disable:4201)
		struct
		{
			float x, y, z, w;
		};
#pragma warning(default:4201)

		Float4A() = default;

		Float4A(const Vec3& xyz, float w)
		{
			v = DirectX::XMVectorSet(
				static_cast<float>(xyz.x),
				static_cast<float>(xyz.y),
				static_cast<float>(xyz.z),
				w
				);
		}
	};

	using Vector = DirectX::XMVECTOR;
	using Matrix = DirectX::XMMATRIX;

	inline Vector _vectorcall ToVector(float x, float y, float z, float w)
	{
		return DirectX::XMVectorSet(x, y, z, w);
	}

	inline Vector _vectorcall ToVector(const Float3& xyz, float w)
	{
		return DirectX::XMVectorSet(xyz.x, xyz.y, xyz.z, w);
	}

	inline Vector _vectorcall ToVector(const Float4& v)
	{
		return DirectX::XMVectorSet(v.x, v.y, v.z, v.w);
	}

	inline Vector _vectorcall ToVector(const Vec3& xyz, float w)
	{
		return DirectX::XMVectorSet(
			static_cast<float>(xyz.x),
			static_cast<float>(xyz.y),
			static_cast<float>(xyz.z),
			w
			);
	}

	inline Vector _vectorcall ToVector(const Vec3& xyz, double w)
	{
		return DirectX::XMVectorSet(
			static_cast<float>(xyz.x),
			static_cast<float>(xyz.y),
			static_cast<float>(xyz.z),
			static_cast<float>(w)
			);
	}

	inline Vector _vectorcall ToVector(const Vec4& v)
	{
		return DirectX::XMVectorSet(
			static_cast<float>(v.x),
			static_cast<float>(v.y),
			static_cast<float>(v.z),
			static_cast<float>(v.w)
			);
	}

	inline Vector _vectorcall ToVector(const ColorF& c)
	{
		return DirectX::XMVectorSet(
			static_cast<float>(c.r),
			static_cast<float>(c.g),
			static_cast<float>(c.b),
			static_cast<float>(c.a)
			);
	}

	inline Vector _vectorcall ToVector(const ColorF& c, float a)
	{
		return DirectX::XMVectorSet(
			static_cast<float>(c.r),
			static_cast<float>(c.g),
			static_cast<float>(c.b),
			a
			);
	}

	/**
	*
	*	@param os
	*	@param v
	*	@return
	*/
	template<typename CharType>
	inline
		std::basic_ostream<CharType>&
		operator << (
		std::basic_ostream<CharType>& os,
		const Vector& v
		)
	{
		return os << CharType('(') << v.m128_f32[0] << CharType(',') << v.m128_f32[1] << CharType(',') << v.m128_f32[2] << CharType(',') << v.m128_f32[3] << CharType(')');
	}

	/**
	*
	*	@param is
	*	@param v
	*	@return
	*/
	template<typename CharType>
	inline
		std::basic_istream<CharType>&
		operator >> (
		std::basic_istream<CharType>& is,
		Vector& v
		)
	{
		CharType unused;
		return is >> unused >> v.m128_f32[0] >> unused >> v.m128_f32[1] >> unused >> v.m128_f32[2] >> unused >> v.m128_f32[3] >> unused;
	}
}
