﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2015 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include "Siv3DFwd.hpp"
# include "SivSound.hpp"

namespace s3d
{
	class DynamicSound
	{
	public:

		class CDynamicSound;
	
		std::unique_ptr<CDynamicSound> m_pSound;

		DynamicSound();

		explicit DynamicSound(Wave&& wave);

		explicit DynamicSound(const FilePath& path);

		DynamicSound(DynamicSound&& sound);

		~DynamicSound();

		DynamicSound& operator = (DynamicSound&& sound);

		void changeTempo(double tempo);

		void changePitchSemitones(double pitchSemitones);

		void play(double fadeinTimeMillisec = 0.0);

		void pause(double fadeoutTimeMillisec = 0.0);

		void stop();

		explicit operator bool() const
		{
			return !isEmpty;
		}

		Property_Get(bool, isEmpty) const;

		Property_Get(bool, isPlaying) const;

		void setVolume(double volume) const;

		void setVolume(double left, double right) const;

		void setAttenuation_dB(double attenuation_dB) const;

		void setAttenuation_dB(double attenuationLeft_dB, double attenuationRight_dB) const;

		const Wave& getWave() const;

		Property_Get(double, volume) const;

		/// <summary>
		/// 左チャンネルの音量
		/// </summary>
		Property_Get(double, volumeLeft) const;

		/// <summary>
		/// 右チャンネルの音量
		/// </summary>
		Property_Get(double, volumeRight) const;


		Property_Get(double, tempo) const;

		Property_Get(double, pitchSemitones) const;

		Property_Get(double, originalPosSec) const;

		Property_Get(uint32, originalPosSample) const;

		Property_Get(double, originalLengthSec) const;

		Property_Get(uint32, originalLengthSample) const;

		Property_Get(uint32, samplingRate) const;
	};
}
