﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2015 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include <memory>
# include "Siv3DFwd.hpp"
# include "SivHandle.hpp"
# include "SivUncopyable.hpp"
# include "SivPropertyMacro.hpp"

namespace s3d
{
	struct IEffect : private Uncopyable
	{
		virtual ~IEffect() = default;

		virtual bool update(double timeSec) = 0;
	};

	class Effect
	{
	private:

		class Handle{};

		using EffectHandle = Siv3DHandle<Handle>;

		std::shared_ptr<EffectHandle> m_handle;

	public:

		Effect();

		~Effect();

		void release();

		explicit operator bool() const { return !isEmpty; }

		Property_Get(HandleIDType, id) const;

		Property_Get(bool, isEmpty) const;

		Property_Get(unsigned, num_effects) const;

		Property_Get(double, speed) const;

		bool operator == (const Effect& effect) const;

		bool operator != (const Effect& effect) const;

		Property_Get(bool, isActive) const;

		void add(std::unique_ptr<IEffect>&& effect) const;
		
		template<typename Emitter, typename... Args>
		void add(Args&&... args)
		{
			add(std::make_unique<Emitter>(std::forward<Args>(args)...));
		}

		void pause() const;

		void resume() const;

		void setSpeed(double speed) const;

		void update() const;
	};
}
