﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2015 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

namespace s3d
{
	inline bool LineInt::operator == (const LineInt& line) const
	{
		return begin == line.begin && end == line.end;
	}

	inline LineInt LineInt::movedBy(int x, int y) const
	{
		return{ begin.movedBy(x, y), end.movedBy(x, y) };
	}

	inline LineInt LineInt::movedBy(const Point& v) const
	{
		return{ begin.movedBy(v), end.movedBy(v) };
	}

	inline LineInt& LineInt::moveBy(int x, int y)
	{
		begin.moveBy(x, y);
		end.moveBy(x, y);
		return *this;
	}

	inline LineInt& LineInt::moveBy(const Point& v)
	{
		begin.moveBy(v);
		end.moveBy(v);
		return *this;
	}

	inline LineInt LineInt::operator + (const Point& v) const
	{
		return{ begin + v, end + v };
	}

	inline LineInt LineInt::operator - (const Point& v) const
	{
		return{ begin - v, end - v };
	}

	inline Point LineInt::vector() const
	{
		return end - begin;
	}

	inline double LineInt::length() const
	{
		return begin.distanceFrom(end);
	}

	inline double LineInt::lengthSq() const
	{
		return begin.distanceFromSq(end);
	}

	inline bool LineInt::operator != (const LineInt& line) const
	{
		return begin != line.begin || end != line.end;
	}

	inline Line Line::movedBy(double x, double y) const
	{
		return{ begin.movedBy(x, y), end.movedBy(x, y) };
	}

	inline Line Line::movedBy(const Vec2& v) const
	{
		return{ begin.movedBy(v), end.movedBy(v) };
	}

	inline Line& Line::moveBy(double x, double y)
	{
		begin.moveBy(x, y);
		end.moveBy(x, y);
		return *this;
	}

	inline Line& Line::moveBy(const Vec2& v)
	{
		begin.moveBy(v);
		end.moveBy(v);
		return *this;
	}

	inline Line Line::operator + (const Vec2& v) const
	{
		return{ begin + v, end + v };
	}

	inline Line Line::operator - (const Vec2& v) const
	{
		return{ begin - v, end - v };
	}

	inline Vec2 Line::vector() const
	{
		return end - begin;
	}

	inline double Line::length() const
	{
		return begin.distanceFrom(end);
	}

	inline double Line::lengthSq() const
	{
		return begin.distanceFromSq(end);
	}
}
