﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2015 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include "Siv3DFwd.hpp"
# include "SivArray.hpp"
# include "SivPointVector.hpp"
# include "SivDirectXMathUtility.hpp"

namespace s3d
{
	/// <summary>
	/// パーティクル
	/// </summary>
	struct Particle
	{
		Float4A pos;

		Float4 scaling;

		Vector color;

		Particle() = default;

		Particle(const Vec3& _pos, double _scaling, const ColorF& _color)
			: pos(_pos, 1.0f)
			, scaling(_scaling, _scaling, 1.0f, 0.0f)
			, color(ToVector(_color)) {}

		Particle(const Vec3& _pos, const Vec2& _scaling, const ColorF& _color)
			: pos(_pos, 1.0f)
			, scaling(_scaling, 1.0f, 0.0f)
			, color(ToVector(_color)) {}
	};

	namespace Graphics3D
	{
		/// <summary>
		/// パーティクルを Forward レンダラ―で描画します。
		/// </summary>
		/// <param name="particles">
		/// パーティクル
		/// </param>
		/// <param name="texture">
		/// 使用するテクスチャ
		/// </param>
		/// <returns>
		/// なし
		/// </returns>
		void DrawParticlesForward(const Array<Particle>& particles, const Texture& texture);

		void DrawParticlesForward(const Array<Particle>& particles, uint32 offset, uint32 count, const Texture& texture);
	}
}
