﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2015 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include "SivTypes.hpp"
# include "SivPointVector.hpp"

namespace s3d
{
	/// <summary>
	/// Perlin Noise 生成
	/// </summary>
	/// <remarks>
	/// リファレンス実装 http://mrl.nyu.edu/~perlin/noise/
	/// </remarks>
	class PerlinNoise
	{
	public:

		explicit PerlinNoise(uint32 seed = 0);

		double noise(double x) const;

		double noise(double x, double y) const;

		double noise(const Vec2& xy) const
		{
			return noise(xy.x, xy.y);
		}

		double noise(double x, double y, double z) const;

		double noise(const Vec3& xyz) const
		{
			return noise(xyz.x, xyz.y, xyz.z);
		}

		double octaveNoise(double x, int32 octaves) const;

		double octaveNoise(double x, double y, int32 octaves) const;

		double octaveNoise(const Vec2& xy, int32 octaves) const
		{
			return octaveNoise(xy.x, xy.y, octaves);
		}

		double octaveNoise(double x, double y, double z, int32 octaves) const;

		double octaveNoise(const Vec3& xyz, int32 octaves) const
		{
			return octaveNoise(xyz.x, xyz.y, xyz.z, octaves);
		}

	private:

		double fade(double t) const;

		double lerp(double t, double a, double b) const;

		double grad(int32 hash, double x, double y, double z) const;

		int p[512];
	};
}
