﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2015 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include <cassert>

namespace s3d
{
	inline Point Point::_get_xx() const
	{
		return{ x, x };
	}

	inline Point Point::_get_xy() const
	{
		return *this;
	}

	inline Point Point::_get_yx() const
	{
		return{ y, x };
	}

	inline Point Point::_get_yy() const
	{
		return{ y, y };
	}

	inline Point Point::operator + () const
	{
		return *this;
	}

	inline Point Point::operator - () const
	{
		return{ -x, -y };
	}

	inline Point Point::operator + (const Point& p) const
	{
		return{ x + p.x, y + p.y };
	}

	inline Point Point::operator - (const Point& p) const
	{
		return{ x - p.x, y - p.y };
	}

	inline Point Point::operator * (int32 s) const
	{
		return{ x*s, y*s };
	}

	inline Vec2 Point::operator * (double s) const
	{
		return{ x*s, y*s };
	}

	inline Point Point::operator * (const Point& p) const
	{
		return{ x*p.x, y*p.y };
	}

	inline Point Point::operator / (int32 s) const
	{
		return{ x / s, y / s };
	}

	inline Vec2 Point::operator / (double s) const
	{
		return{ x / s, y / s };
	}

	inline Point& Point::operator += (const Point& p)
	{
		x += p.x;
		y += p.y;
		return *this;
	}

	inline Point& Point::operator -= (const Point& p)
	{
		x -= p.x;
		y -= p.y;
		return *this;
	}

	inline Point& Point::operator *= (int32 s)
	{
		x *= s;
		y *= s;
		return *this;
	}

	inline Point& Point::operator /= (int32 s)
	{
		assert(s != 0);
		x /= s;
		y /= s;
		return *this;
	}

	inline bool Point::operator == (const Point& p) const
	{
		return x == p.x && y == p.y;
	}

	inline bool Point::operator != (const Point& p) const
	{
		return x != p.x || y != p.y;
	}

	inline double Point::distanceFrom(const Point& p) const
	{
		return std::sqrt(distanceFromSq(p));
	}

	inline double Point::distanceFromSq(const Point& p) const
	{
		return (p.x - x)*(p.x - x) + (p.y - y)*(p.y - y);
	}

	inline Point operator * (int32 s, const Point& p)
	{
		return p*s;
	}

	inline Vec2 operator * (double s, const Point& p)
	{
		return p*s;
	}
}
