﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2015 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include "Siv3DFwd.hpp"
# include "SivUnspecified.hpp"
# include "SivString.hpp"
# include "SivMemoryReader.hpp"
# include "SivImage.hpp"

namespace s3d
{
	enum class QREncodingMode
	{
		Numeric,

		/// <remarks>
		/// 0-9, A-Z, $ % * + - . / : と半角スペース
		/// 小文字と大文字は区別されない
		/// </remarks>
		Alnum,

		Binary,

		Kanji,

		Unknown,
	};

	enum class QRECLevel
	{
		Low,

		Medium,

		Quartile,

		High,
	};

	struct QRData
	{
		String text;

		MemoryReader reader;

		QREncodingMode mode;

		int32 version;

		QRECLevel ecLevel;
	};

	namespace QR
	{
		Optional<int32> MinimumVersion(const String& string, QREncodingMode mode = QREncodingMode::Binary, QRECLevel ec = QRECLevel::Low);

		Optional<int32> MinimumVersion(const void* data, size_t size, QRECLevel ec = QRECLevel::Low);

		inline Optional<int32> MinimumVersion(const MemoryReader& reader, QRECLevel ec = QRECLevel::Low)
		{
			return MinimumVersion(reader.data(), static_cast<size_t>(reader.size()), ec);
		}

		Image Enccode(const String& string, QREncodingMode mode = QREncodingMode::Binary, QRECLevel ec = QRECLevel::Low, const Optional<int32>& version = unspecified);

		Image Enccode(const void* data, size_t size, QRECLevel ec = QRECLevel::Low, const Optional<int32>& version = unspecified);

		inline Image Enccode(const MemoryReader& reader, QRECLevel ec = QRECLevel::Low, const Optional<int32>& version = unspecified)
		{
			return Enccode(reader.data(), static_cast<size_t>(reader.size()), ec, version);
		}

		bool Decode(const Image& image, QRData& data);
	}
}
