﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2015 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include <iostream>
# include "SivPointVector.hpp"

namespace s3d
{
	/// <summary>
	/// レイ
	/// </summary>
	struct Ray
	{
		/// <summary>
		/// レイの始点
		/// </summary>
		Vec3 origin;

		/// <summary>
		/// レイの方向ベクトル
		/// </summary>
		Vec3 direction;

		Ray() = default;

		Ray(const Vec3& _origin, const Vec3& _direction)
			: origin(_origin)
			, direction(_direction) {}

		template<typename Primitive>
		bool intersects(const Primitive& primitive) const
		{
			return Geometry3D::Intersect(*this, primitive);
		}

		bool test(const Sphere& sphere, double& distance) const;

		bool test(const Box& box, double& distance) const;

		bool test(const Triangle3D& triangle, double& distance) const;

		bool test(const Sphere& sphere, Vec3& pos) const;

		bool test(const Box& box, Vec3& pos) const;

		bool test(const Triangle3D& triangle, Vec3& pos) const;
	};

	template<typename CharType>
	inline
	std::basic_ostream<CharType>&
		operator << (
		std::basic_ostream<CharType>& os,
		const Ray& ray
		)
	{
		return	os << CharType('(') << ray.origin << CharType(',') << ray.direction << CharType(')');
	}

	template<typename CharType>
	inline
	std::basic_istream<CharType>&
		operator >> (
		std::basic_istream<CharType>& is,
		Ray& ray
		)
	{
		CharType unused;
		return	is >> unused >> ray.origin >> unused >> ray.direction >> unused;
	}
}
