﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2015 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include "SivPointVector.hpp"
# include "SivMat3x2.hpp"
# include "SivBlendState.hpp"
# include "SivRasterizerState.hpp"
# include "SivDepthStencilState.hpp"
# include "SivSamplerState.hpp"

namespace s3d
{
	struct RenderState
	{
		RenderState() = default;

		RenderState(
			const Mat3x2& _transform2D,
			const Float4& _uvTransform2D,
			const BlendState& _blend,
			const RasterizerState& _rasterizer,
			const DepthStencilState& _depthStencil,
			const SamplerState(&_sampler)[2],
			uint8 _stencilValue
			)
			: transform2D(_transform2D)
			, uvTransform2D(_uvTransform2D)
			, blend(_blend)
			, rasterizer(_rasterizer)
			, depthStencil(_depthStencil)
			, stencilValue(_stencilValue)
		{
			sampler[0] = _sampler[0];
			sampler[1] = _sampler[1];
		}

		Mat3x2 transform2D;

		Float4 uvTransform2D;

		BlendState blend;

		RasterizerState rasterizer;

		DepthStencilState depthStencil;

		SamplerState sampler[2];

		uint8 stencilValue;

		enum class Predefined
		{
			Default2D,
			Default3D,
		};

		RenderState(Predefined predefined);

		const static Predefined Default2D = Predefined::Default2D;

		const static Predefined Default3D = Predefined::Default3D;
	};
}
