﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2015 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include "Siv3DFwd.hpp"
# include "SivPropertyMacro.hpp"
# include "SivPointVector.hpp"
# include "SivTexture.hpp"
# include "SivFont.hpp"

namespace s3d
{
	namespace GUIManager
	{
		const Font& GetDefaultFont();
	}

	/// <summary>
	/// パディング
	/// </summary>
	struct Padding
	{
		Padding() = default;

		Padding(int padding)
			: Padding(padding, padding, padding, padding) {}

		Padding(int _topBottom, int _rightLeft)
			: Padding(_topBottom, _rightLeft, _topBottom, _rightLeft) {}

		Padding(int _top, int _rightLeft, int _bottom)
			: Padding(_top, _rightLeft, _bottom, _rightLeft) {}

		Padding(int _top, int _right, int _bottom, int _left)
			: top(_top)
			, right(_right)
			, bottom(_bottom)
			, left(_left) {}

		int top = 0;

		int right = 0;

		int bottom = 0;

		int left = 0;

		Property_Get(Point, lt) const { return{ left, top }; }

		Property_Get(Point, rb) const { return{ right, bottom }; }
	};

	/// <summary>
	/// マージン
	/// </summary>
	struct Margin
	{
		Margin() = default;

		Margin(int padding)
			: Margin(padding, padding, padding, padding) {}

		Margin(int _topBottom, int _rightLeft)
			: Margin(_topBottom, _rightLeft, _topBottom, _rightLeft) {}

		Margin(int _top, int _rightLeft, int _bottom)
			: Margin(_top, _rightLeft, _bottom, _rightLeft) {}

		Margin(int _top, int _right, int _bottom, int _left)
			: top(_top)
			, right(_right)
			, bottom(_bottom)
			, left(_left) {}

		int top = 0;

		int right = 0;

		int bottom = 0;

		int left = 0;

		Property_Get(Point, lt) const { return{ left, top }; }

		Property_Get(Point, rb) const { return{ right, bottom }; }
	};

	/// <summary>
	/// アライメント
	/// </summary>
	enum class Align
	{
		Left,

		Right,

		Center,
	};

	/// <summary>
	/// 背景画像の繰り返し方法
	/// </summary>
	enum class Repeat
	{
		None,

		ScaleX,

		ScaleY,

		ScaleXY,

		X,

		Y,

		XY,
	};

	/// <summary>
	/// GUI ウィンドウの背景
	/// </summary>
	struct Background
	{
		Optional<Texture> texture;

		Optional<Color> color;

		Optional<Repeat> repeat;
	};

	/// <summary>
	/// ウィジェットのスタイル
	/// </summary>
	struct WidgetStyle
	{
		Padding padding = Padding(0);

		Margin margin = Margin(5);

		Optional<Align> align;

		Background background;

		Optional<int> width;

		Optional<Color> color;

		Font font = GUIManager::GetDefaultFont();
	};

	/// <summary>
	/// ウィジェットの更新と描画のための情報
	/// </summary>
	struct WidgetState
	{
		Point currentPos;

		Point size;

		bool showBoxModel;

		bool active;
	};
}
